# $File: //member/autrijus/DBIx-ReportBuilder/lib/DBIx/ReportBuilder/Clause/Groupby.pm $ $Author: autrijus $
# $Revision: #1 $ $Change: 8712 $ $DateTime: 2003/11/06 15:01:35 $

package DBIx::ReportBuilder::Clause::Groupby;
use base 'DBIx::ReportBuilder::Clause';

1;
